@@ -1,16 +1,17 @@
-## $Id: ORA.pm,v 1.9.1 2013/07/16 14:34:35 Europe/Stirling $
+## $Id: ORA.pm,v 1.5 2008/12/15 10:26:38 Europe/Dublin $
 #
-# Olfactory Receptor Assigner (ORA)
-# Copyright 2007-2013 Bekaert M <michael.bekaert@stir.ac.uk>
+# Olfactory Receptor family Assigner (ORA)
+# Copyright 2007-2008 Bekaert M <michael@batlab.eu>
 #
-# This work is licensed under the Creative Commons Attribution-
-# Noncommercial-Share Alike 3.0 License. To view a copy of this
-# license, visit http://creativecommons.org/licenses/by-nc-sa/3.0/
+# This work is licensed under a Creative Commons GNU General Public
+# License License (GPL). To view a copy of this license,
+# visit http://creativecommons.org/licenses/GPL/2.0/
+# 
 # POD documentation - main docs before the code
 
 =head1 NAME
 
-Bio::ORA - Olfactory Receptor family Assigner (bioperl module)
+Bio::ORA - Olfactory Receptor family Assigner (ORA)
 
 =head1 SYNOPSIS
 
@@ -20,6 +21,9 @@ and orientation of such gene.
 
 Creating the ORA object, eg:
 
+  use Bio::ORA;
+  use Bio::SeqIO;
+
   my $inputstream = Bio::SeqIO->new( -file => 'seqfile', -format => 'fasta' );
   my $seqobj = $inputstream->next_seq();
   my $ORA_obj = Bio::ORA->new( $seqobj );
@@ -33,6 +37,7 @@ Display result in genbank format, eg:
 
   $ORA_obj->show( 'genbank' );
 
+
 =head1 DESCRIPTION
 
 Bio::ORA is a featherweight object for identifying mammalian
@@ -43,15 +48,16 @@ sequence, no intron supported.
 
 See Synopsis above for the object creation code.
 
+
 =head1 DRIVER SCRIPT
 
-  #!/usr/bin/perl
+  #!/usr/bin/perl -w
   use strict;
-  use warnings;
+
   use Bio::Seq;
   use Bio::ORA;
 
-  my $inseq = Bio::SeqIO->new( '-file' => '<' . $ARGV[0], -format => 'fasta' );
+  my $inseq = Bio::SeqIO->new( '-file' => "< $yourfile", -format => 'fasta' );
   while (my $seq = $inseq->next_seq) {
     my $ORA_obj = Bio::ORA->new( $seq );
     if ( $ORA_obj->find() ) {
@@ -61,54 +67,61 @@ See Synopsis above for the object creation code.
     }
   }
 
+
 =head1 REQUIREMENTS
 
 To use this module you may need:
  * Bioperl (L<http://www.bioperl.org/>) modules,
- * HMMER v3 distribution (L<http://hmmer.janelia.org/>) and
- * FASTA r56 distribution (L<ftp://ftp.ebi.ac.uk/pub/software/unix/fasta/>).
+ * HMMER distribution (L<http://hmmer.janelia.org/>) and
+ * FASTA distribution (L<ftp://ftp.ebi.ac.uk/pub/software/unix/fasta/>).
+
+
+=head1 MORE
+ORA home page is now L<http://ora.batlab.eu/>
+
 
 =head1 FEEDBACK
 
 User feedback is an integral part of the evolution of this modules. Send
 your comments and suggestions preferably to author.
 
+
 =head1 AUTHOR
 
-B<Michael Bekaert> (michael.bekaert@stir.ac.uk)
+B<Michael Bekaert> (michael@batlab.eu)
 
 Address:
-     Institute of Aquaculture
-     University of Stirling
-     Stirling
-     Scotland, FK9 4LA
-     UK
+     School of Biology & Environmental Science
+     University College Dublin
+     Belfield, Dublin 4
+     Dublin
+     Ireland
+
+
+=head1 COPYRIGHT
+
+This work is licensed under a Creative Commons GNU General Public
+License License (GPL). The full text of the license can be
+found in the LICENSE file included with this module.
+
 
 =head1 SEE ALSO
 
-perl(1), bioperl web site
+perl(1), fasta, hmmer and bioperl web sites
 
-=head1 LICENSE
-
-Copyright 2007-2013 - Michael Bekaert
-
-This work is licensed under the Creative Commons Attribution-
-Noncommercial-Share Alike 3.0 License. To view a copy of this
-license, visit L<http://creativecommons.org/licenses/by-nc-sa/3.0/>
-or send a letter to Creative Commons, 543 Howard Street, 5th
-Floor, San Francisco, California, 94105, USA
 
 =head1 APPENDIX
 
 The rest of the documentation details each of the object methods. Internal
 methods are usually preceded with a _
 
+
 =cut
 
 package Bio::ORA;
 use strict;
-use vars qw(@ISA $VERSION);
 use File::Temp qw/tempfile/;
+use Bio::Tools::HMMER::Results;
 use Bio::SeqIO;
 use Bio::Seq;
 use Bio::SearchIO;
@@ -116,13 +129,17 @@ use Bio::PrimarySeq;
 use Bio::PrimarySeqI;
 use Bio::Tools::CodonTable;
 
-$VERSION = '1.9.1';
-@ISA     = qw(Bio::Root::Root Bio::Root::IO);
+BEGIN {
+    use vars qw($VERSION @ISA $PATH_REF $PATH_HMM $PATH_TMP);
+    $VERSION     = '1.5';
+    @ISA         = qw(Bio::Root::Root Bio::Root::IO);
+
+    # Default path
+    $PATH_REF = './or.fasta';
+    $PATH_HMM = './or.hmm';
+    $PATH_TMP = '/tmp';
+}
 
-# Default path
-my $PATH_REF = './or.fasta';
-my $PATH_HMM = './or.hmm';
-my $PATH_TMP = '/tmp';
 
 =head2 _findexec
 
@@ -132,7 +149,7 @@ my $PATH_TMP = '/tmp';
  Returns : The full path to the executable.
  Args    : $exec (mandatory) executable to be find.
 
-=cut
+=cut                       
 
 sub _findexec
 {
@@ -169,7 +186,7 @@ sub new
     $self->{'_pfam'} =
       ((defined $ENV{'HMMERDIR'})
         ? $ENV{'HMMERDIR'}
-        : Bio::ORA->_findexec('hmmscan'));
+        : Bio::ORA->_findexec('hmmpfam'));
     $seqobj->throw(
                "die in _initialize, ORA.pm works only on PrimarySeqI objects\n")
       unless ($seqobj->isa("Bio::PrimarySeqI"));
@@ -203,7 +220,7 @@ sub new
  Function: Identify an olfactory receptor protein.
  Returns : boolean.
  Args    : $evalue (optional) set the E-value (expected) threshold.
-             Default is 1e-30,
+             Default is 1e-40,
            $strand(optional) strand where search should be done (1 direct,
              -1 reverse or 0 both). Default is 0,
            $start (optional) coordinate of the first nucleotide. Useful
@@ -217,7 +234,7 @@ sub find
 {
     my ($self, @args) = @_;
     my ($evalue, $strand, $start, $end) = @args;
-    $self->{'_evalue'} = ((defined $evalue) && ($evalue > 0)) ? $evalue : 1e-30;
+    $self->{'_evalue'} = ((defined $evalue) && ($evalue > 0)) ? $evalue : 1e-40;
     $strand = 0 unless ((defined $strand) && ($strand == 1 || $strand == -1));
     $start = 1 unless ((defined $start) && $start > 1);
     $end = $self->{'_seqref'}->length
@@ -286,40 +303,34 @@ sub _what_or
     eval
     {
         my $hmmer =
-           Bio::SearchIO->new(-file => $filename . '.report',
-                                         -format => 'hmmer');
-while( my $result = $hmmer->next_result ) {
-        while( my $hit = $result->next_hit ) {
-            while( my $hsp = $hit->next_hsp ) {
-
-           $self->{'_hmmor'} = (
+          new Bio::Tools::HMMER::Results(-file => $filename . '.report',
+                                         -type => 'hmmpfam');
+        foreach my $domain ($hmmer->each_Domain)
+        {
+            $self->{'_hmmor'} = (
                                  [
-                                  $hsp->evalue(),
-                                  $hit->name,
-                                  substr($result->query_name(), 0, 1),
-                                  $hsp->score(),
-                                  $hsp->start('query'),
-                                  $hsp->end('query'),
+                                  $domain->evalue,
+                                  $domain->hmmname,
+                                  substr($domain->seq_id, 0, 1),
+                                  $domain->bits,
+                                  $domain->start,
+                                  $domain->end,
                                   (
                                    ($strand != 0)
                                    ? $strand
-                                   : ((length($result->query_name()) > 1) ? -1 : 1)
+                                   : ((length($domain->seq_id) > 1) ? -1 : 1)
                                   )
                                  ]
                                 )
               if (!defined $self->{'_hmmor'}[0]
-                  || $hsp->evalue() < $self->{'_hmmor'}[0]);
-            $best = $hsp->evalue()
-              if ((!defined $best) || $hsp->evalue() < $best);
-            $second = $hsp->evalue()
-              if ($hsp->evalue() > $best
-                  && (!defined $second || $hsp->evalue() < $second));
-
-
-            }
+                  || $domain->evalue < $self->{'_hmmor'}[0]);
+            $best = $domain->evalue
+              if ((!defined $best) || $domain->evalue < $best);
+            $second = $domain->evalue
+              if ($domain->evalue > $best
+                  && (!defined $second || $domain->evalue < $second));
         }
-    }
-       $self->{'_hmmor'}[7] = $second
+        $self->{'_hmmor'}[7] = $second
           if (defined $best && defined $self->{'_hmmor'}[0]);
     };
     unlink($filename);
@@ -398,7 +409,7 @@ sub _find_orf
                 $dna                = $2;
                 $stopcodon          = $6;
             }
-            else
+            else 
             {
                 $dna = substr(
                            $seq,
@@ -525,7 +536,7 @@ sub getHits
     my $fasta =
       ((defined $ENV{'FASTADIR'})
         ? $ENV{'FASTADIR'}
-        : $self->_findexec('tfasta36'));
+        : Bio::ORA->_findexec('tfasta34_t'));
     my @hits;
     if ((defined $seq) && (-x $fasta))
     {
@@ -536,7 +547,7 @@ sub getHits
                . $ref . ' '
                . $filename . '> '
                . $filename
-               . '.report') == 0 or return;
+               . '.report');
         eval
         {
             my $in = new Bio::SearchIO(-format => 'fasta',
@@ -560,8 +571,8 @@ sub getHits
             }
         };
         unlink($filename . '.report');
+        unlink($filename);
     }
-    unlink($filename);
     if ($#hits >= 0)
     {
         @hits = sort @hits;
@@ -581,8 +592,8 @@ sub getHits
                 if (
                     (defined $hits[$i + 1])
                     && (
-                        (abs($hitstart_n - $hitend) < 500)
-                        || (abs($hitstart - $hitend_n) < 500)
+                        (abs($hitstart_n - $hitend) < 1000)
+                        || (abs($hitstart - $hitend_n) < 1000)
                         || ( ($hitstart_n < $hitend) && ($hitstart_n > $hitstart) )
                         || ( ($hitend_n < $hitend) && ($hitend_n > $hitstart) )
                         || ( ($hitstart < $hitend_n) && ($hitstart > $hitstart_n) )
@@ -604,13 +615,14 @@ sub getHits
     return @hits;
 }
 
-=head2 fastScan
+=head2 getHits
 
- Title   : fastScan
- Usage   : my @hits = Bio::ORA->fastScan( $seq, $ref );
+ Title   : getHits
+ Usage   : my @hits = Bio::ORA->getHits( $seq, $evalue, $ref );
  Function: Quick localization of ORs (use FASTA).
  Returns : Array of hits start/stop positions.
  Args    : $seq (mandatory) primarySeqI object (dna or rna),
+           $evalue (mandatory) det the E-value threshold,
            $ref (optional) path to fasta reference file, by default ORA
              look at ./or.fasta.
 
@@ -626,7 +638,7 @@ sub fastScan
     my $fasta =
       ((defined $ENV{'FASTADIR'})
         ? $ENV{'FASTADIR'}
-        : $self->_findexec('fastx36'));
+        : Bio::ORA->_findexec('fastx34_t'));
     my @hits;
     if ((defined $seqfile) && (-x $fasta))
     {
@@ -634,20 +646,18 @@ sub fastScan
                . ' -b 1 -d 1 -E 1 -H -Q '
                . $seqfile . ' '
                . $ref . '> '
-               . $filename ) == 0 or return;
+               . $filename );
         eval
         {
             my $in = new Bio::SearchIO(-format => 'fasta',
                                        -file => $filename);
-            my $last;
             while (my $result = $in->next_result)
             {
-                push( @hits, $result->query_name() ) if (!defined $last || $last ne $result->query_name() );
-                $last = $result->query_name();
+                push( @hits, $result->query_name() );
             }
         };
+        unlink($filename);
     }
-    unlink($filename);
     return @hits;
 }
 
@@ -798,7 +808,6 @@ sub show
             print "//\n";
         }
         elsif ( $out eq 'tbl' ) {    # 'Feature Table' output (for tbl2asn, NCBI)
-            my $organism = shift(@args);
             print '>Feature ', $self->{'_seqref'}->display_id, "\n";
             print '<', 1 , "\t>", $self->{'_seqref'}->length(), "\tgene\n";
             print "\t\t\tgene\t", $self->{'_result'}[5], "\n";
@@ -812,7 +821,7 @@ sub show
                 print "\t\t\tproduct\t", $self->{'_result'}[4], "\n";
             }
             print "\n";
-            print STDERR "\n>", $self->{'_seqref'}->display_id, ' [organism=',(defined $organism ? $organism : 'Unknown'),'] ', $self->{'_result'}[4], " gene member, partial cds.\n";
+            print STDERR "\n>", $self->{'_seqref'}->display_id, ' [organism=unknown] ', $self->{'_result'}[4], " gene member, partial cds.\n";
             my $i = 0;
             my $dna = $self->{'_seqref'}->seq();
             while ( length( $dna ) > ( $i * 80 ) ) { print STDERR substr( $dna, ( $i++ * 80 ), 80 ), "\n"; }
@@ -922,13 +931,13 @@ sub _translation
         {
             for (my $k = 0 ; $k < 4 ; $k++)
             {
-                $var[$var_i++] = $table[$i]
+                $var[$var_i++] = $table[$i] 
                   . $table[$j]
                   . $table[$k]
                   if $myCodonTable->is_start_codon(
                                             $table[$i] . $table[$j] . $table[$k]
                   );
-                $var2[$var2_i++] = $table[$i]
+                $var2[$var2_i++] = $table[$i] 
                   . $table[$j]
                   . $table[$k]
                   if $myCodonTable->is_ter_codon(
@@ -941,5 +950,4 @@ sub _translation
     return (join('|', @var), join('|', @var2));
 }
 
-# and that's all the module
 1;
